@@ -1,11 +1,11 @@
 #' Volcano-plot (Statistical Test Outcome versus Relative Change)   
 #'
 #' This type of plot is very common in high-throughput biology, see \href{https://en.wikipedia.org/wiki/Volcano_plot_(statistics)}{Volcano-plot}.
-#' Basically, this plot allows comparing the outcome of a statistical test to the differential of the group means (ie log fold-change),
+#' Basically, this plot allows comparing the outcome of a statistical test to the relative change (ie log fold-change, M-value).
 #' 
-#' In high-throughput biology data are typically already transformed to log2 and thus, the 'M'-value represents a relative change.
-#' Besides, output from statistical testing by \code{\link[wrMisc]{moderTest2grp}} or \code{\link[wrMisc]{moderTestXgrp}} can be directly read to produce Volcano plots for diagnostic reasons.
-#' Please note, that plotting a very number of points in transparency (eg >10000) may take several seconds.
+#' In high-throughput biology data are typically already transformed to log2 and thus, the 'M'-values (obtained by subtrating two group means) represent a relative change.
+#' Output from statistical testing by \code{\link[wrMisc]{moderTest2grp}} or \code{\link[wrMisc]{moderTestXgrp}} can be directly read to produce Volcano plots for diagnostic reasons.
+#' Please note, that plotting a very high number of points (eg >10000) in transparency may take several seconds.
 #'	 
 #' @param Mvalue (numeric or matrix) data to plot; M-values are typically calculated as difference of log2-abundance values and 'pValue' the mean of log2-abundance values;
 #'   M-values and p-values may be given as 2 columsn of a matrix, in this case the argument \code{pValue} should remain NULL 
@@ -16,8 +16,10 @@
 #' @param FCthrs (numeric) Fold-Change threshold (display as line) give as Fold-change and NOT log2(FC), default at 1.5, set to \code{NA} for omitting
 #' @param FdrList (numeric) FDR data or name of list-element
 #' @param FdrThrs (numeric) FDR threshold (display as line), default at 0.05, set to \code{NA} for omitting 
+#' @param FdrType (character) FDR-type to extract if \code{Mvalue} is 'MArrayLM'-object (eg produced by from \code{moderTest2grp} etc);
+#'   if \code{NULL} it will search for suitable fields/values in this order : 'FDR','BH','lfdr' and 'BY'
 #' @param subTxt (character) custom sub-title
-#' @param grayIncrem (logical) if \code{TRUE}, display overlay of points as increased shades of gray
+#' @param grayIncrem (logical) if \code{TRUE}, display overlay of points  (not exceeding thresholds) as increased shades of gray
 #' @param col (character) custom color(s) for points of plot (see also \code{\link[graphics]{par}})
 #' @param pch (integer) type of symbol(s) to plot (default=16) (see also \code{\link[graphics]{par}}) 
 #' @param compNa (character) names of groups compared
@@ -25,25 +27,29 @@
 #' @param cexMa (numeric) font-size of title, as expansion factor (see also \code{cex} in \code{\link[graphics]{par}})
 #' @param cexLa (numeric) size of axis-labels, as expansion factor (see also \code{cex} in \code{\link[graphics]{par}}) 
 #' @param limM (numeric, length=2) range of axis M-values 
-#' @param limp (numeric, length=2) range of axis p-values
-#' @param annotColumn (character) column names of annotation to be extracted (only if \code{Mvalue} is \code{MArrayLM}-object containing matrix $annot)
-#' @param annColor (character or integer) colors for specific groups of annoatation (only if \code{Mvalue} is \code{MArrayLM}-object containing matrix $annot)
+#' @param limp (numeric, length=2) range of axis FDR / p-values
+#' @param annotColumn (character) column names of annotation to be extracted (only if \code{Mvalue} is \code{MArrayLM}-object containing matrix $annot).
+#'   The first entry (typically 'SpecType') is used for different symbols in figure, the second (typically 'GeneName') is used as prefered text for annotating the best points (if \code{namesNBest} allows to do so.)
+#' @param annColor (character or integer) colors for specific groups of annotation (only if \code{Mvalue} is \code{MArrayLM}-object containing matrix $annot)
 #' @param cexPt (numeric) size of points, as expansion factor (see also \code{cex} in \code{\link[graphics]{par}})
 #' @param cexSub (numeric) size of subtitle, as expansion factor (see also \code{cex} in \code{\link[graphics]{par}})
 #' @param cexTxLab (numeric) size of text-labels for points, as expansion factor (see also \code{cex} in \code{\link[graphics]{par}})
-#' @param namesNBest (integer or character) number of best points (by pValue to display names in figure); if 'passThr' all points passing FDR and FC-filtes will be selected
+#' @param namesNBest (integer or character) for display of labels to points in figure: if 'pass','passThr' or 'signif' all points passing thresholds; if numeric (length=1) this number of best points will get labels 
+#'   if the initial object \code{Mvalue} contains a list-element called 'annot' the second of the column specified in argument \code{annotColumn} will be used as text
 #' @param NbestCol (character or integer) colors for text-labels of best points
+#' @param sortLeg (character) sorting of 'SpecType' annotation either ascending ('ascend') or descending ('descend'), no sorting if \code{NULL}
+#' @param NaSpecTypeAsContam (logical) consider lines/proteins with \code{NA} in Mvalue$annot[,"SpecType"] as contaminants (if a 'SpecType' for contaminants already exits)
 #' @param useMar (numeric,length=4) custom margings (see also \code{\link[graphics]{par}})
 #' @param returnData (logical) optional returning data.frame with (ID, Mvalue, pValue, FDRvalue, passFilt) 
 #' @param silent (logical) suppress messages
 #' @param callFrom (character) allow easier tracking of message(s) produced
 #' @param debug (logical) additional messages for debugging 
 #' @return MA-plot only
-#' @seealso (for PCA) \code{\link{plotPCAw}}
+#' @seealso (for PCA) \code{\link{plotPCAw}})
 #' @examples
 #' library(wrMisc)
 #' set.seed(2005); mat <- matrix(round(runif(900),2), ncol=9)
-#' rownames(mat) <- paste0(rep(letters[1:25],each=4), rep(letters[2:26],4))
+#' rownames(mat) <- paste0(rep(letters[1:25], each=4), rep(letters[2:26],4))
 #' mat[1:50,4:6] <- mat[1:50,4:6] + rep(c(-1,1)*0.1,25)
 #' mat[3:7,4:9] <- mat[3:7,4:9] + 0.7
 #' mat[11:15,1:6] <- mat[11:15,1:6] - 0.7
@@ -63,10 +69,10 @@
 #' VolcanoPlotW(tRes, FCth=1.3, FdrThrs=0.2, useComp=2)
 #'  
 #' @export
-VolcanoPlotW <- function(Mvalue, pValue=NULL, useComp=1, filtFin=NULL, ProjNa=NULL, FCthrs=NULL, FdrList=NULL, FdrThrs=NULL,
+VolcanoPlotW <- function(Mvalue, pValue=NULL, useComp=1, filtFin=NULL, ProjNa=NULL, FCthrs=NULL, FdrList=NULL, FdrThrs=NULL, FdrType=NULL,
   subTxt=NULL, grayIncrem=TRUE, col=NULL, pch=16, compNa=NULL, batchFig=FALSE, cexMa=1.8, cexLa=1.1, limM=NULL, limp=NULL,
-  annotColumn=c("SpecType","ProteinName","Accession","Species","Contam","Description"), annColor=NULL, cexPt=NULL, cexSub=NULL, 
-  cexTxLab=0.7, namesNBest=NULL, NbestCol=1, useMar=c(6.2,4,4,2), returnData=FALSE, callFrom=NULL, silent=FALSE,debug=FALSE) {
+  annotColumn=c("SpecType","GeneName","EntryName","Accession","Species","Contam"), annColor=NULL, cexPt=NULL, cexSub=NULL, 
+  cexTxLab=0.7, namesNBest=NULL, NbestCol=1, sortLeg="descend", NaSpecTypeAsContam=TRUE, useMar=c(6.2,4,4,2), returnData=FALSE, callFrom=NULL, silent=FALSE,debug=FALSE) {
   ## MA plot
   ## optional arguments for explicit title in batch-mode
   fxNa <- wrMisc::.composeCallName(callFrom, newNa="VolcanoPlotW")
@@ -75,10 +81,12 @@ VolcanoPlotW <- function(Mvalue, pValue=NULL, useComp=1, filtFin=NULL, ProjNa=NU
   on.exit(graphics::par(opar$cex.main)) 
   on.exit(graphics::par(opar$cex.lab)) 
   on.exit(graphics::par(opar$las)) 
+  plotByFdr <- TRUE  #
   namesIn <- c(deparse(substitute(Mvalue)), deparse(substitute(pValue)), deparse(substitute(filtFin)))
   basRGB <- c(0.3,0.3,0.3)           # grey
   fcRGB <- c(1,0,0)                  # red        for points passing  FC filt line
-  splNa <- annot <- ptType <- colPass <- ptBg <- grpMeans <- pcol <- FDRvalue <- NULL      # initialize  
+  splNa <- annot <- ptType <- colPass <- ptBg <- grpMeans <- pcol <- FDRvalue <- NULL      # initialize
+  multiComp <- TRUE      # initialize  
   if(debug) silent <- FALSE
   if(length(Mvalue) <1) message(" nothing to do, 'Mvalue' seems to be empty !") else  {
     ## data seem valid to make MAplot
@@ -98,33 +106,45 @@ VolcanoPlotW <- function(Mvalue, pValue=NULL, useComp=1, filtFin=NULL, ProjNa=NU
           ## extract 2nd col if result from wrMisc::moderTest2grp() 
           pNa <- rownames(pValue)
           pValue <- as.numeric(pValue[,2])
-          names(pValue) <- pNa 
+          names(pValue) <- pNa
+          multiComp <- FALSE 
         } else {
-          ## need to find corresponding of multiple comparisons
+          ## select corresponding of multiple comparisons
           if(useComp > ncol(pValue)) { useComp <- 1
             if(!silent) message(fxNa," argument 'useComp' for pValues invalid or too high; reset to 1") }
           names(useComp) <- colnames(pValue)[useComp]
           pNa <- rownames(pValue)
           pValue <- as.numeric(pValue[,useComp])
           if(length(pNa) >0) names(pValue) <- pNa }
-      }  ## otherwise spearate pValue was provided
+      }  ## .. otherwise spearate pValue was provided
       ## look for M-values (need to create if not available - using useComp checked when extracting pValue)
-      Melem <- wrMisc::naOmit(match(c("mvalues","mvalue","m"), tolower(names(Mvalue))))   # which list-element
-      Fcol <- wrMisc::naOmit(match(c("fdr","bh","lfdr","by","p.value"), tolower(names(Mvalue))))    # needed for matching means to pair-wise names and for extracting FDR values
-      if(length(Fcol) <1) stop("can't find elment suitable for statistical values'") else Fcol <- Fcol[1]
+      Melem <- wrMisc::naOmit(match(c("mvalues","mvalue","m"), tolower(names(Mvalue))))      # which list-element
+      Fcol <- wrMisc::naOmit(match(if(length(FdrType) <1) c("fdr","bh","lfdr","by","p.value") else FdrType, tolower(names(Mvalue))))    # needed for matching means to pair-wise names and for extracting FDR values
+      if(length(Fcol) <1) stop("Can't find elment suitable for statistical values", if(length(FdrType)==1) c(" to '",FdrType,"'")) else Fcol <- Fcol[1]
+      if(!silent & length(FdrType) <1) message(fxNa,"Using element '",names(Mvalue)[Fcol],"' as FDR-values for plot")
+      if("lfdr" %in% names(Mvalue)[Fcol]) {   # switch from directly plotting FDR-values to uncorrected p-values
+        plotByFdr <- FALSE
+      }
+      
+      ## look for group-means & identify column association to current question/pairwise comparison
+      if("means" %in% names(Mvalue)) {
+        ## identify sammple-groups to comparsison(s) - needed lateron
+        pairwCol <- .sampNoDeMArrayLM(Mvalue, useComp, lstMeans="means", lstP=Fcol,callFrom=fxNa,silent=silent) 
+        grpMeans <- cbind(mean1=Mvalue$means[,pairwCol[1]], mean2=Mvalue$means[,pairwCol[2]])  
+        ## are all group-means needed (for exporting) ??
+      } else warning("Could not find suitable field '$means' in '",namesIn[1],"'")    
+      
       if(length(Melem) >0) {            ## M-values are available
         Mvalue$Mval <- Mvalue[[Melem]]
         if(length(dim(Mvalue$Mval)) >0) if(ncol(Mvalue$Mval) >1) {          
           if(useComp[1] > ncol(Mvalue$Mval)) { if(!silent) message(fxNa," 'useComp' is too high, set to 1"); useComp <- 1 }
           Mvalue$Mval <- Mvalue$Mval[,useComp]}
-      } else {                               #  need to construct M-values based on means
+      } else {                               # need to construct M-values based on means
         if("means" %in% names(Mvalue)) {
-          ## identify sammple-groups to comparsison(s)
-          meanCol <- .sampNoDeMArrayLM(Mvalue, useComp, lstMeans="means", lstP=Fcol) 
-          grpMeans <- cbind(mean1=Mvalue$means[,meanCol[1]], mean2=Mvalue$means[,meanCol[2]])
+          ## construct Mvalue based on means (only one/current pairwise comparison needed)
           Mvalue$Mval <- grpMeans[,2] - grpMeans[,1]                           
           Melem <- which(names(Mvalue)=="Mval")                # update
-        } else stop("Could not find suitable field '$means' in '",namesIn[1],"' to construct M-values !")    
+        } else stop("Can't construct M-values since suitable field '$means' missing in '",namesIn[1],"' !")    
       }
       ## now one can check if 'pValue' & Mvalue match
       chPM <- length(pValue) >0 & length(as.numeric(pValue)) == length(as.numeric(Mvalue[[Melem]]))
@@ -134,8 +154,7 @@ VolcanoPlotW <- function(Mvalue, pValue=NULL, useComp=1, filtFin=NULL, ProjNa=NU
              pValue <- as.numeric(Mvalue[[pcol[1]]][,useComp])          
           } else stop("'pValue' & 'Mvalue' don't match")  
         } else stop("'pValue' & 'Mvalue' don't match (no field in MArrayLM-object available)") 
-      }
-       
+      }       
       ## extract FDR from MArrayLM-object 
       if(length(FdrList) <1) {
         ## no explicit pValue, try to extract from MArrayLM-object (Mvalue)
@@ -144,7 +163,8 @@ VolcanoPlotW <- function(Mvalue, pValue=NULL, useComp=1, filtFin=NULL, ProjNa=NU
           if(length(dim(FDRvalue)) >0) if(colnames(FDRvalue)[1]=="(Intercept)" & ncol(FDRvalue) >1) {
             pNa <- rownames(FDRvalue)
             FDRvalue <- as.numeric(FDRvalue[,2])
-            names(FDRvalue) <- pNa }
+            names(FDRvalue) <- pNa 
+            multiComp <- FALSE }
         } else {
           FDRvalue <- stats::p.adjust(pValue) 
           if(!silent) message(fxNa,"No FDR data found, generating BH-FDR")}
@@ -160,20 +180,28 @@ VolcanoPlotW <- function(Mvalue, pValue=NULL, useComp=1, filtFin=NULL, ProjNa=NU
       if(length(filtFin) <1 | identical(filtFin, FALSE)) {
         Fcol <- wrMisc::naOmit(match(c("filtfin","filter","filt","finfilt"), tolower(names(Mvalue))))
         filtFin <- if(length(Fcol) >0) Mvalue[[Fcol[1]]] else rep(TRUE,length(pValue))
+        if(length(dim(filtFin)) >1) filtFin <- filtFin[,useComp]        
       }
       ## recuperate $annot if present and use for symbol
       if("annot" %in% names(Mvalue)) {
-        useAnnCol <- match(annotColumn,colnames(Mvalue$annot))      
-        if(!is.na(useAnnCol[1])) {                     # annotation (for multiple groups) exists
-          ptType <- Mvalue$annot[,useAnnCol[1]]
-          if(length(pch) < length(pValue) & length(unique(ptType)) >1) {
+        useAnnCol <- match(annotColumn, colnames(Mvalue$annot))      
+        if(!is.na(useAnnCol[1])) {                         # annotation (for multiple groups) exists
+          ptType <- Mvalue$annot[,useAnnCol[1]]            # SpecType
+          chNA <- is.na(ptType)
+          ## associate NAs from 'SpecType' in ptType with conta ?
+          if(NaSpecTypeAsContam) {
+            chConta <- tolower(ptType) %in% c("contaminant","contam","conta","cont")
+            if(any(chConta)) ptType[which(is.na(ptType))] <- unique(ptType[which(chConta)])[1]}          
+          if(any(is.na(ptType))) ptType[which(chNA)] <- "NA" 
+          if(length(pch) < length(pValue) & length(unique(wrMisc::naOmit(ptType))) >1) {
             if(length(pch) >1 & !silent) message(fxNa," (invalid pch) using default 'pch' oriented by $annot and starting from 15")
-            pch <- 14 + as.numeric(as.factor(ptType))
+            pch <- 14 + as.integer(as.factor(ptType))
           } 
           useAnnCol <- wrMisc::naOmit(useAnnCol)
           annot <- Mvalue$annot[,useAnnCol] 
+          if(annotColumn[1] %in% colnames(annot)) annot[,annotColumn[1]] <- ptType           
       } }
-      if(length(pch)==1) pch <- rep(as.integer(pch), length(pValue))        
+      if(length(pch)==1) pch <- rep(as.integer(pch), length(pValue))
       
       ## recuperate M values (& dismiss rest of MArrayLM-object)  
       Mvalue <- Mvalue$Mval
@@ -182,7 +210,7 @@ VolcanoPlotW <- function(Mvalue, pValue=NULL, useComp=1, filtFin=NULL, ProjNa=NU
         if(length(MNa) >0) names(Mvalue) <- MNa }
       ## additional check for length 
       chpM <- length(Mvalue)==length(pValue)  
-      if(!chpM & !silent) message("p- and M- values have different length !!  (M=",length(Mvalue)," vs p=",length(pValue),")")
+      if(!chpM & !silent) message(fxNa,"trouble ahead ? p- and M- values have different length !!  (M=",length(Mvalue)," vs p=",length(pValue),")")
 
       ## done with extracing MArrayLM-object    
       if(!silent) message(fxNa,"Successfully extracted  ",length(Mvalue)," Mvalues and  ",length(pValue)," pValues", if(length(annot) >0) c(" plus anotation"))      
@@ -195,10 +223,8 @@ VolcanoPlotW <- function(Mvalue, pValue=NULL, useComp=1, filtFin=NULL, ProjNa=NU
         pNa <- rownames(pValue)
         pValue <- as.numeric(pValue[,ncol(pValue)] )  
         names(pValue) <- pNa} 
-      if(length(FdrList) <1) {
-        FDRvalue <- NULL
-      } else FDRvalue <- FdrList  
-    } 
+      FDRvalue <- if(length(FdrList) <1) NULL else FdrList
+    }
     
     ## need to introduce -log10 to pValue
     chNA <- is.na(pValue)
@@ -218,13 +244,19 @@ VolcanoPlotW <- function(Mvalue, pValue=NULL, useComp=1, filtFin=NULL, ProjNa=NU
         if(length(Mvalue) != length(pValue)) stop("p- and M- values have different length, but no names to match !!  (M=",length(Mvalue)," vs p=",length(pValue),")")
       }
     if(length(grpMeans) <1) grpMeans <- matrix(rep(NA,2*length(Mvalue)), ncol=2, dimnames=list(names(Mvalue),c("mean1","mean2")))
-    
+   
     ## start creating merged data for plot (& export)
     merg <- if(length(annot) >0) data.frame(ID=NA, grpMeans, Mvalue=Mvalue, pValue=pValue, FDR=if(length(FDRvalue) >0) FDRvalue else rep(NA,length(pValue)), 
-      filtFin=filtFin, annot, stringsAsFactors=FALSE) else {
-      data.frame(ID=NA, grpMeans, Mvalue=Mvalue, pValue=pValue, FDR=FDRvalue, filtFin=filtFin, stringsAsFactors=FALSE) }
+      filtFin=filtFin, annot, pch=pch, stringsAsFactors=FALSE) else {
+      data.frame(ID=NA, grpMeans, Mvalue=Mvalue, pValue=pValue, FDR=FDRvalue, filtFin=filtFin, pch=pch, stringsAsFactors=FALSE) }
     if(length(names(Mvalue)) >0) merg[,1] <- names(Mvalue) else {if(length(names(pValue)) >0) merg[,1] <- names(pValue)}
+    ## replace NA in 'SpecType' by 'NA'
 
+    if(annotColumn[1] %in% colnames(merg)) { chNa <- is.na(merg[,annotColumn[1]])     # replace NAs in col "SpecType" by "NA"
+      if(any(chNa)) merg[which(chNa),annotColumn[1]] <- "NA"
+    } else { merg <- cbind(merg, rep(1,nrow(merg)))            # add colum for 'SpecType'
+      colnames(merg)[ncol(merg)] <- annotColumn[1] }
+    
     ## adjust col & pch
     if(!any(c(1,length(Mvalue)) %in% length(pch))) {
       if(!silent) message(fxNa,"argument 'pch' should be either length=1 or correspond to length of data, reset to default=16")
@@ -232,7 +264,7 @@ VolcanoPlotW <- function(Mvalue, pValue=NULL, useComp=1, filtFin=NULL, ProjNa=NU
     if(length(col) >1 & length(col) <length(Mvalue)) {
       if(!silent) message(fxNa,"argument 'col' should be either length=1 or correspond to length of data, reset to default=NULL")
       col <- NULL }
-       
+
     ## prepare/integrate FILTERING
     if(length(filtFin) >0) {
       ## if filtFin is matrix use each line with min 1 instance of TRUE,
@@ -243,7 +275,7 @@ VolcanoPlotW <- function(Mvalue, pValue=NULL, useComp=1, filtFin=NULL, ProjNa=NU
       } else if(length(filtFin)==nrow(merg)) merg[,"filtFin"] <- filtFin        # no proof that order of filtFin is correct
     } else filtFin <- rep(TRUE, nrow(merg)) 
     if(debug) message(fxNa," ++ DONE extracting columns : ",wrMisc::pasteC(colnames(merg),quo="'"))
-    
+
     ## apply filtering
     msg <- " data provided in 'Mvalue' and 'pValue' "
     if(!silent & nrow(merg) < round(length(Mvalue)/10)) message(" .. note : less than 10% of",msg," were matched") else {
@@ -259,13 +291,21 @@ VolcanoPlotW <- function(Mvalue, pValue=NULL, useComp=1, filtFin=NULL, ProjNa=NU
         if(!silent) message(fxNa," filtered (based on 'filtFin') from ",length(filtFin)," to  ",nrow(merg)," lines")
       }
     } else filtFin <- rep(TRUE, nrow(merg))
+    
+    ## sort merg, so that legend always gets constructed the same order, ascending ('ascend') or descending ('descend')    
+    sortLeg <- if(identical(sortLeg,"ascend")) FALSE else {if(identical(sortLeg,"descend")) TRUE else NULL}
+    if(length(sortLeg) ==1 & annotColumn[1] %in% colnames(merg)) merg <- merg[order(merg[,annotColumn[1]], decreasing=sortLeg),]     
+     
+    ## update ..
     nIDco <- sum(c("ID","nredID","uniqID") %in% colnames(merg))                   #  number of heading columns in 'merg'
     Mvalue <- as.numeric(if("Mvalue" %in% colnames(merg)) merg[,"Mvalue"] else merg[,nIDco+1])
     pValue <- as.numeric(if("pValue" %in% colnames(merg)) merg[,"pValue"] else {
       if(length(dim(Mvalue)) >0) merg[,ncol(Mvalue) +nIDco +1] else merg[,nIDco+2]})
     if("Lfdr" %in% colnames(merg)) FdrList <- merg[,"Lfdr"] else {
       if("lfdr" %in% colnames(merg)) FdrList <- merg[,"lfdr"]}
-
+    pch <- merg[,"pch"]            # update
+    ptType <- if(annotColumn[1] %in% colnames(merg)) merg[,annotColumn[1]] else rep(1,nrow(merg))     # update "SpecType"
+    
     ## prepare for  plotting
     if(is.null(cexSub)) cexSub <- cexLa +0.05  
     xLab <- "M-value (log2 fold-change)"
@@ -273,94 +313,133 @@ VolcanoPlotW <- function(Mvalue, pValue=NULL, useComp=1, filtFin=NULL, ProjNa=NU
       if(!is.null(compNa)) c(compNa[1]," vs ",compNa[2])), collapse=" ")    # but what title if batchFig=NULL & compNa=NULL -> only "Volcano-plot"
     if(length(FCthrs) <1) FCthrs <- 1.5 
     if(length(FdrThrs) <1) FdrThrs <- 0.05 
+
     ## count no of passing
-    passFC <- if(length(FCthrs) ==1 & !any(is.na(FCthrs))) abs(Mvalue) > log2(FCthrs) else merg[,"filtFin"]      ## convert FCthrs to log2
+    passFC <- if(length(FCthrs) ==1 & !any(is.na(FCthrs))) abs(merg[,"Mvalue"]) >= log2(FCthrs) else merg[,"filtFin"]      ## convert FCthrs to log2
     passFdr <- if(length(FdrThrs) ==1 & !any(is.na(FdrThrs))) {merg[,"FDR"] <= FdrThrs} else merg[,"filtFin"]
     passAll <- merg[,"filtFin"] & passFC & passFdr
+
     chNA <- is.na(passAll)                              # passFdr may contain NAs
     if(any(chNA)) passAll[which(chNA)] <- FALSE 
     if(debug) message(fxNa,"  ",sum(passFC,na.rm=TRUE)," passing FCthrs ; ",sum(passFdr,na.rm=TRUE)," passing FdrThrs ; combined ",sum(passAll,na.rm=TRUE))
     ## color for points passing filter
+    if(length(col) >0) if(length(col) != nrow(merg)) { col <- NULL
+      if(!silent) message(fxNa," invalid entry for 'col', should be of length=",nrow(merg),", resetting to default")}
     if(length(col) <1) {
       alph <- sort(c(0.14, round(0.6/log10(length(Mvalue)),2), 0.8))[2]       # alph <- round(12/sqrt(nrow(eBayesLst$pValue)),2)
-          alph2 <- max(round(4/(10 +sum(filtFin)^0.25),2), alph) 
-          alph3 <- max(round(1.3/(10 +sum(filtFin)^0.25),2), alph +0.05)                 # alternative for alph2 
-      useCol <- if(grayIncrem) grDevices::rgb(0.35,0.35,0.35,alph) else grDevices::rgb(0.7,0.7,0.7)
+      alph2 <- sort(c(round(7/(5 +sum(passAll)^0.7),2), alph,0.9))[2]                   # for points passing thresholds
+      useCol <- if(grayIncrem) grDevices::rgb(0.35,0.35,0.35,alph) else grDevices::rgb(0.7,0.7,0.7)  # basic color
       useCex <- if(length(cexPt) >0) cexPt else max(round(0.8 + 2/(1 +sum(filtFin, na.rm=TRUE))^0.28,2), 1.1)
-      colPass <- grDevices::rgb(0.8,0.01,0.01, alph2)                 # (default) red
-      whCol <- rep(1, length(Mvalue))
-      if(any(passAll)) whCol[which(passAll)] <- 2                     # assign color for those passing
-      useCol <- c(useCol,colPass)[whCol]  
+      chCol <- unique(merg[, annotColumn[1]])      # check how many different colors may be needed
+      chNaC <- is.na(chCol)
+      if(any(chNaC)) chCol[which(chNaC)] <- "NA"    
+      if(length(annColor) >0) {colPass <- annColor} else if(length(chCol) >4) {
+        colPass <- cbind(red=c(141,72,90,171, 220,253,244,255), green=c(129,153,194,221, 216,174,109,0), blue=c(194,203,185,164, 83,97,67,0))       
+        colPass <- grDevices::rgb(red=colPass[,1], green=colPass[,2], blue=colPass[,3], alph2, maxColorValue=255)
+        if(length(chCol) >8) { colPass <- c(colPass, rep(colPass[8], length(chCol) -8))
+          if(!silent) message(fxNa," > 8 different groups found, using 8th color after 7th group")}
+      } else colPass <- grDevices::rgb(c(0.95,0.2,0,0.75), c(0.15,0.2,0.9,0.35), c(0.15,0.95,0,0.8), alph2)    # red, blue, green, purple (luminosity adjusted) 
+      useCol <- rep(useCol[1], nrow(merg))         # fuse basic gray to colors for different types
+
+      ## integrate names of annColor as order of colPass
+      if(length(names(annColor)) >0) {
+        uniTy <- unique(merg[which(passAll),annotColumn[1]])
+        colPass <- colPass[match(names(annColor), uniTy)]
+      }
+      
+      
+      
+      ## assign color for those passing
+      if(any(passAll)) useCol[which(passAll)] <- colPass[if(length(unique(merg[which(passAll),annotColumn[1]])) >1) .levIndex(merg[which(passAll),annotColumn[1]]) else rep(1,sum(passAll))]  # assign colors for those passing 
     } else useCol <- col
     ## adjust fill color for open symbols
     chPch <- pch %in% c(21:25)
-    if(any(chPch)) {ptBg <- col ; col[which(chPch)] <- grDevices::rgb(0.2,0.2,0.2, max(alph,0.4)) }
-    
+    if(any(chPch)) { ptBg <- useCol
+      ptBg[which(chPch)] <- useCol[which(chPch)]    # background color for filled symbols
+      useCol[which(chPch)] <- 1                     # contour as black
+    }
+
     ## main graphic
     graphics::par(mar=c(6.5,4,4,2), cex.main=cexMa, las=1)
-    graphics::plot(Mvalue, pValue, pch=pch, cex=useCex, main=tit1, ylab="- log10 p-value (uncorrected)", col=useCol, xlab=xLab, cex.lab=cexLa, xlim=limM,ylim=limp, pt.bg=ptBg)
-    sTxt <- if(length(subTxt) ==1) subTxt else {if(length(names(useComp)) >0) names(useComp) else paste("useComp",useComp)}
-    sTxt <- paste0(sTxt,"; n=",length(Mvalue),
-      if(!all(is.na(c(FCthrs,FdrThrs)))) paste("; ",sum(passAll, na.rm=TRUE)," (red) points passing",
-        if(!is.na(FCthrs)) paste0(" (FCthr=", as.character(FCthrs),", ") else " (", paste0("FdrThrs=",as.character(FdrThrs),")")))
+    ## rather directly plot FDR
+    graphics::plot(Mvalue, if(plotByFdr) -1*log10(merg[,"FDR"]) else merg[,"pValue"], pch=pch, cex=useCex, main=tit1, 
+      ylab=if(plotByFdr) "- log10 FDR" else "- log10 p-value (uncorrected)", col=useCol, xlab=xLab, cex.lab=cexLa, xlim=limM,ylim=limp, pt.bg=ptBg)    
+    
+    sTxt <- if(length(subTxt) ==1) subTxt else { if(multiComp) paste0(if(length(names(useComp)) >0) names(useComp) else c("useComp=",useComp),"; ",collapse="")}
+    sTxt <- paste0(sTxt,"n=",length(Mvalue),
+      if(!all(is.na(c(FCthrs,FdrThrs)))) paste(";",sum(passAll, na.rm=TRUE),"(color) points passing",
+        if(!is.na(FCthrs)) paste0("(FCthr=", as.character(FCthrs),", ") else " (", paste0("FdrThrs=",as.character(FdrThrs),")")))
     graphics::mtext(sTxt,cex=0.75,line=0.2)
     if(!all(is.na(c(FCthrs,FdrThrs)))) { 
       if(debug) message(fxNa," n=",length(Mvalue),"  FCthrs=",as.character(FCthrs),"  filt.ini=", sum(filtFin, na.rm=TRUE),
         "  passAll=",sum(passAll,na.rm=TRUE)," ; range Mva ",wrMisc::pasteC(signif(range(Mvalue,na.rm=TRUE),3))," ;  alph=",alph,"  useCex=",useCex,"  alph2=",alph2)
-      graphics::abline(v=c(-1,1)*log2(FCthrs) +c(0.01,-0.02), col=grDevices::rgb(0.87,0.72,0.72), lty=2) }
-    if(!any(is.na(FdrThrs))) {
-      dThr <- FDRvalue -FdrThrs
-      dThr2 <- which(dThr <0) 
-      if(length(dThr2) >0) {
-        limP <- dThr2[which(dThr[dThr2] ==max(dThr[dThr2], na.rm=TRUE))]
-        limP <- min(pValue[limP], na.rm=TRUE)
-        graphics::abline(h=limP -0.015, col=grDevices::rgb(0.87,0.72,0.72), lty=2) }
-    }
-
+      graphics::abline(v=c(-1,1)*(log2(FCthrs) + diff(graphics::par("usr")[1:2])/500), col=grDevices::rgb(0.87,0.72,0.72), lty=2) }
+    if(sum(passFdr) >0) {
+      if(plotByFdr) {
+        graphics::abline(h=-1*log10(max(merg[passAll,"FDR"], na.rm=TRUE)) -diff(graphics::par("usr")[3:4])/400, col=grDevices::rgb(0.87,0.72,0.72), lty=2)
+      } else { 
+        graphics::mtext("Note, that FDR and p-value may not correlate perfectly, thus points may appear at good p-value but finally don't get retained",line=-1.4,cex=0.7)
+        pRa <- range(merg[which(passFdr),"pValue"], na.rm=TRUE)
+        graphics::abline(h=pRa[1] +diff(graphics::par("usr")[3:4])/400, col=grDevices::rgb(0.87,0.72,0.72), lty=2) }}
+    
     ## add names to best points
     if(length(namesNBest) >0) { 
-      if(identical(namesNBest,"passThr") | identical(namesNBest,"signif")) namesNBest <- sum(passAll) 
+      if(any(sapply( c("pass","passThr","passFDR","signif"), identical, namesNBest))) namesNBest <- sum(passAll)
       if(!is.integer(namesNBest)) namesNBest <- try(as.integer(namesNBest))
-      tmP <- as.numeric(merg[,"pValue"])
-      if(any(!passAll)) tmP[which(!passAll)] <- min(tmP,na.rm=TRUE) -1
-      useLi <- order(tmP, decreasing=TRUE)[1:namesNBest]      
-      xOffs <- signif(abs(diff(if(length(limM)==2) limp else range(Mvalue, na.rm=TRUE) ))/130,3)
-      yOffs <- signif(abs(diff(if(length(limp)==2) limp else range(pValue, na.rm=TRUE) ))/90,3)
-      ## alternative names to display
-      dispNa <- merg[useLi,1] 
-      if(length(annotColumn) >2) if(annotColumn[2] %in% colnames(annot)) {          # if available, use 2nd annotColumn for text lables, default annot-column 'ProteinName'
-        dispNa <- merg[useLi,annotColumn[2]] }
-      if(any(is.na(dispNa))) message(fxNa," Some names for points to highlight are and won't get displayed !")  
-      if(length(NbestCol) <1) NbestCol <- 1
-      graphics::text(Mvalue[useLi] +xOffs, pValue[useLi] +yOffs, dispNa, cex=cexTxLab, col=NbestCol, adj=0)
-    }          
+      if(namesNBest >0 & any(passAll)) {      
+        useLi <- if(any(!passAll)) which(passAll) else 1:nrow(merg)
+        tmP <- as.numeric(merg[useLi,"pValue"])
+        names(tmP) <- rownames(merg)[useLi]
+        ## look for more informative names to display
+        if(length(annot) >0) {
+          proNa <- annot[match(names(tmP), rownames(annot)), annotColumn[2]]   # normally 'Description'
+          chNa <- is.na(proNa)
+          if(!all(chNa)) names(tmP)[which(!chNa)] <- proNa[which(!chNa)]
+        }        
+        useL2 <- order(tmP, decreasing=TRUE)[1:min(namesNBest,sum(passAll))]      
+        xOffs <- signif(diff(graphics::par("usr")[1:2])/170,3)
+        yOffs <- signif(diff(graphics::par("usr")[3:4])/90,3)
+        noNa <- if(is.null(names(tmP[useL2]))) 1:length(tmP) else which(is.na(names(tmP)[useL2]))
+        if(length(noNa) >0 & all(annotColumn %in% colnames(merg))) names(tmP)[useL2[noNa]] <- merg[useLi[useL2[noNa]], wrMisc::naOmit(match(annotColumn[-1], colnames(merg)))[1]]
+        if(length(NbestCol) <1) NbestCol <- 1
+        if(is.null(names(tmP[useL2]))) {if(!silent) message(fxNa," No names available for displaying names of best in plot")
+        } else { displTx <- wrMisc::naOmit(match(annotColumn[2:4], colnames(merg)))
+          displTx <- if(length(displTx) >0) cbind(merg[useLi[useL2], displTx], rownames(merg)[useLi[useL2]]) else as.matrix(useLi[useL2])
+          chNa <- rowSums(!is.na(displTx)) >0
+          if(any(chNa)) displTx[which(chNa),1] <- "unknown"          
+          displTx <- apply(displTx,1, function(x) wrMisc::naOmit(x)[1])         
+          graphics::text(Mvalue[useLi[useL2]] +xOffs, yOffs -1*log10(merg[useLi[useL2], if(plotByFdr) "FDR" else "pValue"]),
+          names(tmP)[useL2], cex=cexTxLab, col=NbestCol, adj=0) } 
+      }
+    }              
 
-    ## custom colors for significant points
-    if(length(annColor) >0 & sum(passAll) >0) {  ## replot points passing thresholds according to colors given
-      useLi <- which(passAll)
-      if(length(annColor) ==length(pValue)) useCol <- annColor[useLi] else {
-        useCol <- annColor[as.numeric(as.factor(merg[,annotColumn[1]]))][useLi]
-      } 
-      graphics::points(Mvalue[useLi], pValue[useLi], pch=pch[useLi], cex=useCex, col=useCol, pt.bg=ptBg)    
-    } else annColor <- NULL
-    
     ## legend (if multiple symbols)
+    pch[which(is.na(pch))] <- -2
     ch1 <- unique(pch)
     if(length(ch1) >1) {
-      legLab <- wrMisc::naOmit(if(length(ptType) >0) unique(ptType) )
-      legLoc <- wrGraph::checkForLegLoc(cbind(Mvalue, pValue), sampleGrp=legLab, showLegend=FALSE)
-      ##
-      legCex <- stats::median(c(useCex,cexTxLab,1.2),na.rm=TRUE)
-      ptCol <- if(length(colPass) >0) {if(length(annColor) >0 & length(annotColumn) >0) annColor[order(unique(wrMisc::naOmit(merg[,annotColumn[1]])))] else colPass} else useCol[1]
-      graphics::legend(legLoc$loc, legend=rev(legLab), col=rev(ptCol), text.col=1, pch=rev(wrMisc::naOmit(unique(pch))), if(length(ptBg) >0) pt.bg=ptBg, 
-        cex=legCex, pt.cex=1.2*legCex, xjust=0.5, yjust=0.5)        # as points
+      legInd <- which(!duplicated(merg[which(passAll), annotColumn[1]], fromLast=FALSE))
+      legPch <- pch[which(passAll)[legInd]]
+      legCol <- useCol[which(passAll)[legInd]]
+      legBg <- ptBg[which(passAll)[legInd]]
+      if(alph2 <1) {legCol <- substr(legCol,1,7); legBg <- substr(legBg,1,7)}  # reset to no transparency
+      legLab <- merg[which(passAll)[legInd], annotColumn[1]]
+      chNa <- is.na(legLab)
+      if(any(chNa)) legLab[chNa] <- "NA"
+      legOr <- if(length(legLab) >1) order(legLab) else 1   # not used so far 
+      legLoc <- checkForLegLoc(cbind(Mvalue, pValue), sampleGrp=legLab, showLegend=FALSE)
+      legCex <- stats::median(c(useCex,cexTxLab,1.2), na.rm=TRUE)
+      graphics::legend(legLoc$loc, legend=legLab, col=legCol, text.col=1, pch=legPch, if(length(ptBg) >0) pt.bg=ptBg, cex=legCex, pt.cex=1.2*legCex, xjust=0.5, yjust=0.5)  # as points
     }
+
   ## export results
-  if(returnData) cbind(merg[,1:5], FDRvalue=FDRvalue, merg[,-(1:5)]) 
+  if(returnData) {
+    merg <- merg[,-1*c(1,ncol(merg))]        # remove col 'ID' 'redundant' & 'pch'
+    annCo <- wrMisc::naOmit(match(annotColumn, colnames(merg)))
+    if(length(annCo) >0) cbind(merg[,annCo],  merg[,-annCo]) else merg }
   } }
      
 #' @export
-.sampNoDeMArrayLM <- function(MArrayObj, useComp, groupSep="-",lstMeans="means",lstP="BH",silent=FALSE,callFrom=NULL) {
+.sampNoDeMArrayLM <- function(MArrayObj, useComp, groupSep="-", lstMeans="means", lstP="BH", silent=FALSE, callFrom=NULL) {
   ## locate sample index from index or name of pair-wise comparisons in list or MArrayLM-object
   fxNa <- wrMisc::.composeCallName(callFrom, newNa=".sampNoDeMArrayLM")
   errMsg <- c("argument 'MArrayObj' is ","empty","doesn't contain the list-element needed  ('",lstMeans,"') !")
@@ -376,9 +455,43 @@ VolcanoPlotW <- function(Mvalue, pValue=NULL, useComp=1, filtFin=NULL, ProjNa=NU
     ## convert/locate names to index 
     if(is.character(useComp) & length(grep("[[:alpha:]]",useComp)) >0) useComp <- wrMisc::naOmit(match(useComp, MArrayObj[[lstP]] ))
     if(length(useComp) <1) stop("argument 'useComp' is empty or can't locate in comparison-names")
+    if(length(useComp) >1) { useComp <- useComp[1]; message(fxNa," using only 1st instance of argument 'useComp'")}
+    if(useComp > ncol(MArrayObj[[lstP]])) {useComp <- 1; message(fxNa," argument 'useComp' is too high (max ",ncol(MArrayObj[[lstP]]),"), resetting to 1")}
     ## main
     out <- if(ncol(MArrayObj[[lstP]])==2 & colnames(MArrayObj[[lstP]])[1] =="(Intercept)") 1:2 else {
       wrMisc::matchSampToPairw(grpNa=colnames(MArrayObj[[lstMeans]]), pairwNa=colnames(MArrayObj[[lstP]])[useComp], sep=groupSep,silent=silent,callFrom=fxNa)}
     if(length(useComp)==1) out <- as.integer(out)}
   out }
     
+#' @export
+.levIndex <- function(dat,asSortedLevNa=FALSE) {
+  ## transform levels into index; should get integrated to wrMisc
+  if(asSortedLevNa) out <- as.integer(as.factor(dat)) else {
+    out <- dat
+    levU <- wrMisc::naOmit(unique(out))     # levels in orig order (non-alpahbetical)
+    for(i in 1:length(levU)) out[which(out==levU[i])] <- i
+    out <- as.integer(out)}
+  out }
+
+#' @export
+.colorByPvalue <- function(x,br=NULL,col=NULL,asIndex=FALSE) {
+  ## this function should ultimately get incormporated to wtMisc::colorAccording2 as option "pValue" or "FDR"
+  ## colors significant (***,**,*) as red-tones, 0.05-0.075 as pale orange, others as blue-tones
+  ## 'br' .. custom breaks (used with cut)
+  ## 'col' .. custom colors (must be of length(br) -1)
+  ## 'asIndex' .. optional returning of index to \code{col} instead of final color designation
+  fxNa <- ""
+  if(length(br) >0) br <- sort(unique(wrMisc::naOmit(br)))
+  if(length(col) != length(br) -1 & length(br) >0) { br <- col <- NULL
+    message(fxNa, "entries for 'col' and 'br' don't match, ignoring") }
+  if(length(col) <1) {
+    col <- grDevices::colorRampPalette((RColorBrewer::brewer.pal(n=7, name="RdYlBu")))(22)[c(2:4,7,17:20)]  
+    if(length(col) != length(br)) br <- sort(c(1e-20,10^(-3:0), 0.05,0.075,0.2,0.3))
+    }
+  out <- if(asIndex) as.numeric(cut(x, br)) else col[as.numeric(cut(x, br))]
+  if(asIndex) out <- as.numeric(cut(x, br)) else {
+    out <- col[as.numeric(cut(x, br))]
+    chNa <- is.na(x)
+    if(any(chNa)) out[which(chNa)] <- grDevices::grey(0.4) }
+  out }
+  
